import QtQuick 2.12
import QtQuick.Controls 2.12
import QtQuick.Layouts  1.12
import QtQuick.Dialogs 1.3
import QtQml 2.12

import com.st.motorcontrol.qml 1.0
import com.st.motorcontrol.motorcontrolregister 1.0
import "."



ScrollView {
    id:scrollview
    width: 1300



       Frame {
        id: surface

        /* Motor Control Application parameters */
        property int minimumSpeed: -2000											/* EURO2021 <- */
        property int maximumSpeed: 2000												/* EURO2021 <- */
        property int startupSpeed: 1250												/* EURO2021 <- */
        property int nomimalVBus: 24
        property int maximumVBus: 30
        property int minimumVBus: 10
        property int maximumPower: 10
        property int maximumTemperature: 100
        property string registerFileDescription: "RegListSTMV2_EURO2021.json"
        																			/* EURO2021 <- */
        property string version : "v1.5"

		/* EURO2021 \|/ */

		/* Hands-On02 task 1 * /
		property int targetSpeed: 1
		property int myConnection: 0

		Timer {
			id: timerEuro
			interval: 1000;
			running: false;
			repeat: true

			onTriggered: {
				if ((mcRegBank.getRegisterByName( "SPEED_MEAS" ).value === 0) && (cyclingButton.text === "Stop cycling")) {
            		cyclingButton.text = "Start cycling"
                    timerEuro.stop()
				}
				else {
					timerEuro.interval = speedDelay.value * 1000
					if (surface.targetSpeed === 1) {
						mcRegBank.getRegisterByName( "SPEED_RAMP" ).setValue( [ rampTargetSpeed.value, speedRampDuration.value ] )
						surface.targetSpeed = 2
					}
					else {
						mcRegBank.getRegisterByName( "SPEED_RAMP" ).setValue( [ rampTargetSpeed2.value, speedRampDuration.value ] )
						surface.targetSpeed = 1
					}
				}
			}
		}

        Connections {
            target: mcMain
            onConnectionStatusChanged: {
               if ( status === 1  ) { 			// mcMain.CONNECTED
                 surface.myConnection = 1
               } else if ( status === 3  ) { 	// mcMain.TIMEOUT
                 surface.myConnection = 0
               } else if ( status === 2  ) { 	// mcMain.DISCONNECTED
                 surface.myConnection = 0
               } else {
                 surface.myConnection = 0
               }
            }
        }
		/* EURO2021 /|\ */


        font.pixelSize: 12

        RowLayout {
            id: mainGrid

               spacing: 2

            ColumnLayout {
                id: globalControlColumn

                Layout.alignment: Qt.AlignTop | Qt.AlignLeft

                McStatusBox {
                    id:statusgroupBox
                    title: "Status"

                    Layout.fillWidth: true
                }

                GroupBox {
                    id: motorControlGroupBox

                    Layout.fillWidth: true

                    title: "Control"

                    ColumnLayout {
                        spacing: 2

                        anchors.fill: parent

                        McRegComboBox {
                            id: controlMode

                            Layout.fillWidth: true

                            regName: "CONTROL_MODE"
                            label: "Mode"
                        }

                        Button {
                            id: startMotorButton

                            text: "Start"
                            Layout.fillWidth: true

                            onClicked: mcApplication.startMotor( 1 )
                        }

                        Button {
                            id: stopMotorButton

                            text: "Stop"
                            Layout.fillWidth: true

                            onClicked: mcApplication.stopMotor( 1 )
                        }

                        Button {
                            id: stopRampButton

                            text: "Stop ramp"
                            Layout.fillWidth: true


                            onClicked: mcApplication.stopRamp( 1 )
                        }
                    } // ColumnLayout
                } // motorControlGroupBox

                GroupBox {
                    id: appConfigGroupBox

                    title: "Configuration"

                    Layout.fillWidth: true

                    ColumnLayout {
                        Button {
                            id: appConfigButton
                            text: "Application Configuration"

                            Layout.fillWidth: true

                            onClicked: configurationPopup.open()

                            Popup {
                                id: configurationPopup

                                x: parent.width / 2
                                    y: -parent.height * 5

                                modal: true
                                closePolicy: Popup.CloseOnEscape | Popup.CloseOnPressOutside

                                GridLayout {
                                    columns: 3

                                    Label { text: "Minimum application speed:" }

                                    McSpinBox {
                                        id: minSpeed;

                                        from: -2000000000;
                                        to: 2000000000;

                                        value: surface.minimumSpeed;

                                        widthHint: "- 2000000000"
                                    }

                                    Label { text: "RPM" }

                                    Label { text: "Maximum application speed:" }

                                    McSpinBox {
                                        id: maxSpeed;

                                        from: -2000000000;
                                        to: 2000000000;

                                        value: surface.maximumSpeed;

                                        widthHint: "- 2000000000"
                                    }

                                    Label { text: "RPM" }

                                    Label { text: "Target startup speed:" }

                                    McSpinBox {
                                        id: startupSpeed;

                                        from: -2000000000;
                                        to: 2000000000;

                                        value: surface.startupSpeed;

                                        widthHint: "- 2000000000"
                                    }

                                    Label { text: "RPM" }

                                    Label { text: "Maximum Bus Voltage:" }

                                    McSpinBox {
                                        id: maxVBus;

                                        from: 0;
                                        to: 2000000000;

                                        value: surface.maximumVBus;

                                        widthHint: "2000000000"
                                    }

                                    Label { text: "Volts" }

                                    Label { text: "Maximum Motor Power:" }

                                    McSpinBox {
                                        id: maxPower;

                                        from: 0;
                                        to: 2000000000;

                                        value: surface.maximumPower;

                                        widthHint: "2000000000"
                                    }



                                    Label { text: "Watts" }



                                } // GridLayout

                                onClosed: {
                                    surface.minimumSpeed = minSpeed.value
                                    surface.maximumSpeed = maxSpeed.value
                                    surface.startupSpeed = startupSpeed.value
                                    surface.maximumVBus = maxVBus.value
                                    surface.maximumPower = maxPower.value
                                }


                            } // configurationPopup
                        } // appConfigButton

                        Button {
                            id: advancedModeButton

                            Layout.fillWidth: true

                            text: "Advanced Configuration"
                            checkable: true
                            onCheckedChanged: {

                            }
                        }
                    }
                } // appConfigGroupBox
                Label{
                    id: labelversion
                    text: surface.version

                    //y : surface.height - 20
                    //y : mainGrid.height - 20
                    //anchors.bottom: surface.bottom
                    x:0
                }

            } // globalControlColumn


            Frame {
             id: surface2
            ColumnLayout {
                id: globalAppColumn
                Layout.alignment: Qt.AlignTop | Qt.AlignLeft
                TabBar {
                    id: mainTabBar
                    implicitWidth: 250
                    TabButton { text: "Application" }
                    TabButton { text: "Rev-up" }
                }
                StackLayout {
                    currentIndex: mainTabBar.currentIndex
                    Layout.fillWidth: true
                    RowLayout {
                        id: mainAppTabGrid

                        StackLayout {
                            id: controlsStackLayout

							/* EURO2021 \|/ */
                            /* currentIndex: controlMode.value */
                            currentIndex: (surface.myConnection === 0) ? 1 : controlMode.value
                            /* EURO2021 /|\ */

                            Layout.alignment: Qt.AlignTop

                            /* Torque Control */
                            ColumnLayout {
                                id: torqueControlColumn

                                GroupBox {
                                    id : torqueControlGroupBox

                                    Layout.alignment: Qt.AlignTop | Qt.AlignLeft
                                    title: "Torque Control"

                                    ColumnLayout {
                                        McRegCircularGauge {
                                            id: currentTorqueGauge

                                            regName: "I_Q_MEAS"

                                            width: 302
                                            height: width

                                            from: -32768
                                            to: 32767
                                        } // currentTorqueGauge

                                        McRegView {
                                            id: currentTorqueView

                                            regName: "I_Q_MEAS"

                                            label: "Torque"
                                            widthHint: "- 88,888"

                                            Layout.alignment: Qt.AlignHCenter
                                        } // currentTorqueView

                                        ToolSeparator { orientation: Qt.Horizontal }

                                        Slider {
                                            id: torqueRefSlider

                                            property real torqueRefRegVal: mcRegBank.getRegisterByName( "I_Q_REF" ).value
                                            property bool loopBreaker: false

                                            from: -32768
                                            to: 32767
                                            stepSize: (to-from)/600
                                            snapMode: Slider.NoSnap

                                            wheelEnabled: true

                                            Layout.fillWidth: true

                                            background: Rectangle {
                                                x: torqueRefSlider.leftPadding
                                                y: torqueRefSlider.topPadding + torqueRefSlider.availableHeight / 2 - height / 2
                                                //implicitWidth: 200
                                                implicitHeight: 6
                                                width: torqueRefSlider.availableWidth
                                                height: implicitHeight
                                                radius: 2
                                                color: "transparent"

                                                Rectangle {
                                                    x: 0
                                                    y: parent.height / 2 - height / 2

                                                    width: parent.width
                                                    height: 6
                                                    radius: 3
                                                    color: palette.midlight
                                                }

                                                Rectangle {
                                                    x: Math.min( torqueRefSlider.visualPosition, 0.5 ) * parent.width
                                                    y: parent.height / 2 - height / 2
                                                    width: Math.abs( 0.5 - torqueRefSlider.visualPosition) * parent.width
                                                    height: 6
                                                    color: enabled ? palette.highlight : palette.mid
                                                    radius: 3
                                                }
                                            } // background

                                            handle: Rectangle {
                                                x: torqueRefSlider.leftPadding + torqueRefSlider.visualPosition * (torqueRefSlider.availableWidth - width)
                                                y: torqueRefSlider.topPadding + torqueRefSlider.availableHeight / 2 - height / 2
                                                implicitWidth: 20
                                                implicitHeight: 20
                                                radius: 10
                                                color: enabled ? (Math.abs( torqueRefSlider.visualPosition - 0.5 ) > 0.45 ? "red" : (Math.abs( torqueRefSlider.visualPosition - 0.5 ) < 0.1 ? "orange" : palette.button) ) : palette.button
                                                // speedRefSlider.pressed ? "#f0f0f0" : "#f6f6f6"
                                                border.color: palette.dark //"#bdbebf"
                                                border.width: 1
                                            } // handle

                                            onValueChanged: {
                                                if ( ! loopBreaker ) {
                                                    mcRegBank.getRegisterByName( "TORQUE_RAMP" ).setValue( [ value, 0 ] )
                                                }
                                                loopBreaker = false
                                            } // ValueChanged

                                            onTorqueRefRegValChanged: {
                                                loopBreaker = true
                                                value = torqueRefRegVal
                                                loopBreaker = false
                                            }

                                            Component.onCompleted: {
                                                value =  torqueRefRegVal
                                                mcRegBank.getRegisterByName("I_Q_REF" ).setPolling( true )
                                            }
                                        } // torqueRefSlider

                                        McValueView {
                                            label: "Torque Reference"
                                            unit: "s16A"
                                            tooltip: "Use this slider to set the torque generated by the motor"
                                            widthHint: "- 32,768"
                                            decimals: 0

                                            value: torqueRefSlider.value

                                            Layout.alignment: Qt.AlignHCenter
                                        }
                                    } // ColumnLayout
                                } // torqueControlGroupBox

                                GroupBox {
                                    id: torqueRampGroupBox
                                    title: "Torque Ramp"

                                    Layout.column: 1
                                    Layout.row: 1
                                    Layout.fillWidth: true
                                    Layout.alignment: Qt.AlignTop

                                    GridLayout {
                                        columns: 3
                                        anchors.centerIn: parent

                                        Label { text: "Target Torque" }

                                        SpinBox {
                                            id: rampTargetTorque

                                            property real rampTorqueRefRegVal: mcRegBank.getRegisterByName( "I_Q_REF" ).value

                                            from: surface.minimumSpeed
                                            to: surface.maximumSpeed

                                            contentItem: TextInput {
                                                text: rampTargetTorque.value
                                                readOnly: ! rampTargetTorque.editable
                                                selectByMouse: true

                                                font: rampTargetTorque.font
                                                horizontalAlignment: Qt.AlignRight

                                                Keys.onPressed: {
                                                    if ((event.key === Qt.Key_Enter) || (event.key === Qt.Key_Return)) {
                                                        mcRegBank.getRegisterByName( "TORQUE_RAMP" ).setValue( [ Number(text), torqueRampDuration.value ] )
                                                    }
                                                }
                                            }

                                            editable: true
                                            wheelEnabled: true

                                            onRampTorqueRefRegValChanged: {
                                                value = rampTorqueRefRegVal
                                            }
                                        }

                                        Label { text: "s16A" }

                                        Label { text: "Duration" }

                                        SpinBox {
                                            id: torqueRampDuration

                                            from: 0
                                            to: 2000000000
                                            stepSize: 100

                                            contentItem: TextInput {
                                                text: torqueRampDuration.value
                                                readOnly: ! torqueRampDuration.editable
                                                selectByMouse: true

                                                font: torqueRampDuration.font
                                                horizontalAlignment: Qt.AlignRight

                                                Keys.onPressed: {
                                                    if ((event.key === Qt.Key_Enter) || (event.key === Qt.Key_Return)) {
                                                        mcRegBank.getRegisterByName( "TORQUE_RAMP" ).setValue( [ rampTargetTorque.value, Number(text) ] )
                                                    }
                                                }
                                            }

                                            editable: true
                                            wheelEnabled: true
                                        }

                                        Label { text: "ms" }

                                        Button {
                                            text: "Execute"

                                            Layout.columnSpan: 3
                                            Layout.fillWidth: true

                                            onClicked: mcRegBank.getRegisterByName( "TORQUE_RAMP" ).setValue( [ rampTargetTorque.value, torqueRampDuration.value ] )
                                        }
                                    } // GridLayout
                                } // torqueRampGroupBox



                                // *******************************************

                            } // torqueControlColumn

                            /* Speed Control */
                            ColumnLayout {
                                id: speedControlColumn

                                GroupBox {
                                    id : speedControlGroupBox

                                    title: "Speed Control"

                                    Layout.alignment: Qt.AlignTop | Qt.AlignLeft

                                    ColumnLayout {
                                        McRegCircularGauge {
                                            id: currentSpeedGauge

                                            regName: "SPEED_MEAS"

                                            width: 302
                                            height: width

                                            from: surface.minimumSpeed
                                            to: surface.maximumSpeed
                                        }

                                        McRegView {
                                            regName: "SPEED_MEAS"

                                            label: "Mechanical Speed"
                                            widthHint: "- 88888"

                                            Layout.alignment: Qt.AlignHCenter
                                        }

                                        ToolSeparator { orientation: Qt.Horizontal }

                                        Slider {
                                            id: speedRefSlider

                                            property real rampTargetSpeedRegVal: {
                                                if (mcRegBank.getRegisterByName( "SPEED_RAMP" ))
                                                {
                                                    if (mcRegBank.getRegisterByName( "SPEED_RAMP" ).values.length > 0)
                                                        return mcRegBank.getRegisterByName( "SPEED_RAMP" ).values[0]
                                                }
                                                return 0
                                            }
                                            property bool loopBreaker: false

                                            from: surface.minimumSpeed
                                            to: surface.maximumSpeed
                                            stepSize: (to-from)/600
                                            snapMode: Slider.NoSnap

                                            wheelEnabled: true

                                            Layout.fillWidth: true

                                            background: Rectangle {
                                                x: speedRefSlider.leftPadding
                                                y: speedRefSlider.topPadding + speedRefSlider.availableHeight / 2 - height / 2
                                                //implicitWidth: 200
                                                implicitHeight: 6
                                                width: speedRefSlider.availableWidth
                                                height: implicitHeight
                                                radius: 2
                                                color: "transparent"

                                                Rectangle {
                                                    x: 0
                                                    y: parent.height / 2 - height / 2

                                                    width: parent.width
                                                    height: 6
                                                    radius: 3
                                                    color: palette.midlight
                                                }

                                                Rectangle {
                                                    x: Math.min( speedRefSlider.visualPosition, 0.5 ) * parent.width
                                                    y: parent.height / 2 - height / 2
                                                    width: Math.abs( 0.5 - speedRefSlider.visualPosition) * parent.width
                                                    height: 6
                                                    color: enabled ? palette.highlight : palette.mid
                                                    radius: 3
                                                }
                                            } // background

                                            handle: Rectangle {
                                                x: speedRefSlider.leftPadding + speedRefSlider.visualPosition * (speedRefSlider.availableWidth - width)
                                                y: speedRefSlider.topPadding + speedRefSlider.availableHeight / 2 - height / 2
                                                implicitWidth: 20
                                                implicitHeight: 20
                                                radius: 10
                                                color: enabled ? (Math.abs( speedRefSlider.visualPosition - 0.5 ) > 0.45 ? "red" : (Math.abs( speedRefSlider.visualPosition - 0.5 ) < 0.1 ? "orange" : palette.button) ) : palette.button
                                                // speedRefSlider.pressed ? "#f0f0f0" : "#f6f6f6"
                                                border.color: palette.dark //"#bdbebf"
                                                border.width: 1
                                            }

                                            onValueChanged: {
                                console.log( "speedRefSlider onValueChanged..." )
                                                if ( ! loopBreaker ) {
                                    console.log( "speedRefSlider onValueChanged programming into FW" )
                                                    mcRegBank.getRegisterByName( "SPEED_RAMP" ).setValue( [ value, 0  ] )
                                                    speedRef.value = Math.round(value)
                                                }

                                console.log( "speedRefSlider onValueChanged... done" )
                                                loopBreaker = false
                                            }

                                            onRampTargetSpeedRegValChanged: {
                                                loopBreaker = true
                                console.log( "speedRefSlider onRampTargetSpeedRegValChanged about to set " + rampTargetSpeedRegVal )
                                                value = rampTargetSpeedRegVal
                                console.log( "speedRefSlider onRampTargetSpeedRegValChanged did set " )
                                                loopBreaker = false
                                            }

                                            Component.onCompleted: value = surface.startupSpeed
                                        }

                                        McRegView {
                                            id : speedRef
                                            label: "Speed Reference"
                                            unit: "RPM"
                                            tooltip: "Use this slider to set the rotation speed of the motor"
                                            widthHint: {
                                                console.log ("minimumSpeed = " + surface.minimumSpeed)
                                                if (surface.minimumSpeed)
                                                    return Number(surface.minimumSpeed).toLocaleString( Qt.locale("C"), 'f', decimals )
                                            }

                                            regName: "SPEED_REF"


                                            //value: Math.round(speedRefSlider.value)

                                            Layout.alignment: Qt.AlignHCenter
                                            //Component.onCompleted: console.log ( "Speed Reference : " + Number(minimumSpeed).toLocaleString( Qt.locale("C"), 'f', decimals ) )
                                        }
                                    } // ColumnLayout
                                } // speedControlGroupBox

                                GroupBox {
                                    id: speedRampGroupBox
                                    title: "Speed Ramp"

                                    Layout.fillWidth: true
                                    Layout.alignment: Qt.AlignTop

                                    GridLayout {
                                        columns: 3
                                        anchors.centerIn: parent

                                        Label { text: "Target Speed" }

                                        SpinBox {
                                            id: rampTargetSpeed

                                            // TBD: change
                                            property real rampTargetSpeedRegVal: {
                                                if (mcRegBank.getRegisterByName( "SPEED_RAMP" ))
                                                {
                                                    if (mcRegBank.getRegisterByName( "SPEED_RAMP" ).values.length > 0)
                                                        return mcRegBank.getRegisterByName( "SPEED_RAMP" ).values[0]
                                                }
                                                return 0
                                            }

                                            from: surface.minimumSpeed
                                            to: surface.maximumSpeed

                                            contentItem: TextInput {
                                                text: rampTargetSpeed.value
                                                readOnly: ! rampTargetSpeed.editable
                                                selectByMouse: true

                                                font: rampTargetSpeed.font
                                                horizontalAlignment: Qt.AlignRight

                                                Keys.onPressed: {
                                                    if ((event.key === Qt.Key_Enter) || (event.key === Qt.Key_Return)) {
                                                        mcRegBank.getRegisterByName( "SPEED_RAMP" ).setValue( [ Number(text), speedRampDuration.value  ] )
                                                    }
                                                }
                                            }

                                            editable: true
                                            wheelEnabled: true

                                            onRampTargetSpeedRegValChanged: {
                                                value = rampTargetSpeedRegVal
                                            }

                                            Component.onCompleted: value = surface.startupSpeed
                                        }

                                        Label { text: "RPM" }

                                        Label { text: "Duration" }

                                        SpinBox {
                                            id: speedRampDuration

                                            from: 0
                                            to: 2000000000
                                            stepSize: 100

                                            contentItem: TextInput {
                                                text: speedRampDuration.value
                                                readOnly: ! speedRampDuration.editable
                                                selectByMouse: true

                                                font: speedRampDuration.font
                                                horizontalAlignment: Qt.AlignRight

                                                Keys.onPressed: {
                                                    if ((event.key === Qt.Key_Enter) || (event.key === Qt.Key_Return)) {
                                                        mcRegBank.getRegisterByName( "SPEED_RAMP" ).setValue( [ rampTargetSpeed.value, Number(text) ] )
                                                    }
                                                }
                                            }

                                            editable: true
                                            wheelEnabled: true
                                        }

                                        Label { text: "ms" }

                                        Button {
                                            text: "Execute Speed Ramp"

                                            Layout.columnSpan: 3
                                            Layout.fillWidth: true

                                            onClicked: {
                                                console.log("Execute Speed Ramp fired!!!")
                                                mcRegBank.getRegisterByName( "SPEED_RAMP" ).setValue( [ rampTargetSpeed.value, speedRampDuration.value ] )
                                            }

                                        }

										/* EURO2021 \|/ */

										/* Hands-On02 task 2 * /
							            Rectangle {
							            	Layout.columnSpan: 3
							                Layout.fillWidth: true
							                height: 1
							                color: surface.palette.button
							            }

							            Label { text: "Target Speed 2" }

							            SpinBox {
							                id: rampTargetSpeed2
							                from: surface.minimumSpeed
							                value: 750
							                to: surface.maximumSpeed

							                contentItem: TextInput {
							                    text: rampTargetSpeed2.value
							                    readOnly: ! rampTargetSpeed2.editable
							                    selectByMouse: true

							                    font: rampTargetSpeed2.font
							                    horizontalAlignment: Qt.AlignRight
							                }

							                editable: true
							                wheelEnabled: true
							            }
							            Label { text: "RPM" }

							            Label { text: "Delay" }

							            SpinBox {
							                id: speedDelay

							                from: 1
							                value: 2
							                to: 2000000
							                stepSize: 1

							                contentItem: TextInput {
							                    text: speedDelay.value
							                    readOnly: ! speedDelay.editable
							                    selectByMouse: true

							                    font: speedDelay.font
							                    horizontalAlignment: Qt.AlignRight
							                }

							                editable: true
							                wheelEnabled: true
							            }
							            Label { text: "s" }

							            Button {
							            	id: cyclingButton
							                text: "Start cycling"

							                Layout.columnSpan: 3
							                Layout.fillWidth: true

							                onClicked: {
							                	if(timerEuro.running === false) {
							                		text = "Stop cycling"
								                    timerEuro.interval = 1
								                    timerEuro.start()
								                }
								                else {
							                		text = "Start cycling"
								                    timerEuro.stop()
								                }
							                }

							            }
										/* EURO2021 /|\ */

                                    } // GridLayout
                                } // speedRampGroupBox
                            } // speedControlColumn

                        } // controlsStackLayout

                        ColumnLayout{
                            id:measureAndDebugLayout
                            Layout.alignment: Qt.AlignTop

                            GroupBox {
                                id: otherMeasures

                                title: "Measures"

                                Layout.alignment: Qt.AlignTop


                                ColumnLayout {
                                    spacing: 2

                                    RowLayout{


                                            McRegGaugeVertical{
                                                id: gaugeVerticalVbus
                                                regName: "BUS_VOLTAGE"
                                                label: "VBUS"
                                                maximumValue: surface.maximumVBus
                                                minimumValue: 0
                                                maxErrorThreshold:  surface.maximumVBus
                                                maxWarningThreshold:  surface.maximumVBus - (surface.maximumVBus - surface.nomimalVBus)/2

                                                minWarningThreshold:  surface.minimumVBus + (surface.nomimalVBus - surface.minimumVBus)/2
                                                minErrorThreshold:  surface.minimumVBus



                                            }
                                            McRegGaugeVertical{
                                                id: gaugeVerticalTemp
                                                regName: "HEATS_TEMP"
                                                label: "TEMP"
                                                maximumValue: surface.maximumTemperature
                                                maxErrorThreshold:  surface.maximumTemperature*90/100
                                                maxWarningThreshold:  surface.maximumTemperature*70/100
                                                tickmarkStepSize: 10


                                            }
                                            McRegGaugeVertical{
                                                id: gaugeVerticalPower
                                                regName: "MOTOR_POWER"
                                                label: "POWER"
                                                maximumValue: surface.maximumPower
                                                maxErrorThreshold:  surface.maximumPower*90/100
                                                maxWarningThreshold:  surface.maximumPower*50/100

                                                tickmarkStepSize: 1

                                            }

                                    }


                                    Frame{
                                        Layout.fillWidth: true
                                        StackLayout {
                                            id: sideMeasuresStack

                                            Layout.alignment: Qt.AlignHCenter
                                            Layout.fillWidth: true

                                            currentIndex: controlMode.value

                                            ColumnLayout {
                                                id: speedMeasureColumn

                                                Layout.alignment: Qt.AlignHCenter
                                                Layout.fillWidth: true

                                                Label {
                                                    text: "Speed"
                                                    Layout.alignment: Qt.AlignHCenter
                                                }

                                                McRegCircularGauge {
                                                    width: 100
                                                    height: width
                                                    Layout.alignment: Qt.AlignHCenter

                                                    regName: "SPEED_MEAS"
                                                    from: surface.minimumSpeed
                                                    to: surface.maximumSpeed


                                                }

                                                McRegView {
                                                    regName: "SPEED_MEAS"
                                                    label: "Speed"
                                                    widthHint: "- 32,768"
                                                    Layout.alignment: Qt.AlignHCenter
                                                    Layout.fillWidth: true
                                                }

                                                //ToolSeparator { orientation: Qt.Horizontal }

                                                McRegView {
                                                    regName: "I_D_MEAS"
                                                    label: "Id"
                                                    widthHint: "- 32,768"
                                                    Layout.alignment: Qt.AlignHCenter
                                                    Layout.fillWidth: true
                                                }
                                            } // speedMeasureColumn

                                            ColumnLayout {
                                                id: torqueMeasuresColumn

                                                Layout.alignment: Qt.AlignHCenter
                                                Layout.fillWidth: true

                                                Label {
                                                    text: "Currents"

                                                    Layout.alignment: Qt.AlignHCenter
                                                }


                                                McRegCircularGauge {
                                                    width: 100
                                                    height: width

                                                    regName: "I_Q_MEAS"
                                                    from: -40000
                                                    to: 40000

                                                    Layout.alignment: Qt.AlignHCenter
                                                }

                                                McRegView {
                                                    regName: "I_Q_MEAS"
                                                    label: "Iq"
                                                    widthHint: "- 32,768"
                                                    Layout.alignment: Qt.AlignHCenter
                                                }

                                                McRegView {
                                                    regName: "I_D_MEAS"
                                                    label: "Id"
                                                    widthHint: "- 32,768"
                                                    Layout.alignment: Qt.AlignHCenter
                                                }
                                            } // torqueMeasuresColumn
                                        }
                                    }

                                } // other measures column Layout

                            } // otherMeasures
                            GroupBox {
                                id: debugGroupBox

                                Layout.fillWidth: true


                                property var dacSignals: mcRegBank.getHFRegNameListFromJson()

                                title: "Debug / DAC"


                                ColumnLayout {
                                    Layout.fillWidth: true
                                    RowLayout {
                                        id:dac1RowLayout
                                        Layout.fillWidth: true

                                        Label { text: "DAC 1" }

                                        ComboBox {
                                            property var reg: mcRegBank.getRegisterByName("DAC_OUT1")
                                            model: debugGroupBox.dacSignals
                                            Layout.fillWidth: true
                                            Layout.minimumWidth: 150
                                            onCurrentIndexChanged: {
                                                var regfromName = mcRegBank.getRegisterByName(currentText)
                                                if (regfromName)
                                                    reg.value = regfromName.regId
                                            }
                                            Component.onCompleted: mcRegBank.getRegisterByName("DAC_OUT1").setPolling(true)
                                        }
                                    }
                                    RowLayout {
                                        id:dac2RowLayout
                                        Layout.fillWidth: true

                                        Label { text: "DAC 2" }

                                        ComboBox {
                                            property var reg: mcRegBank.getRegisterByName("DAC_OUT2")
                                            model: debugGroupBox.dacSignals
                                            Layout.fillWidth: true

                                            onCurrentIndexChanged: {
                                                var regfromName = mcRegBank.getRegisterByName(currentText)
                                                if (regfromName)
                                                    reg.value = regfromName.regId
                                                }
                                            Component.onCompleted: mcRegBank.getRegisterByName("DAC_OUT2").setPolling(true)
                                            }
                                        }
                                    }

                            }//Debug GroupBox
                        } // ColumnLayout

                        McGroupBoxFWConfig
                        {
                            id:groupBoxconfig
                        }

//                        GroupBox
//                        {
//                            id:groupBoxconfig
//                            title: "MC Configuration"
//                            ColumnLayout
//                            {

//                                Rectangle
//                                {
//                                    Connections {
//                                        target: mcMain
//                                        onConnectionStatusChanged: {
//                                            console.log( "onConnectionStatusChanged: " + status )

//                                            if ( status === 1 /*mcMain.CONNECTED*/ ) {
//                                                console.log( "onConnectionStatusChanged: CONNECTED" )
//                                                config1.buildText( config1Reg.reg.values )
//                                                config2.buildText( config2Reg.reg.values )
//                                                config3.buildText( config3Reg.reg.values )

//                                            } else if ( status === 3 /*mcMain.TIMEOUT*/ ) {
//                                                console.log( "onConnectionStatusChanged: TIMEOUT" )
//                                            } else if ( status === 2 /*mcMain.DISCONNECTED*/ ) {
//                                                console.log( "onConnectionStatusChanged: DISCONNECTED" )
//                                                config1.buildText( ["-","-","-","-","-"] )
//                                                config2.buildText( ["-","-","-","-","-"] )
//                                                config3.buildText( ["-","-","-","-","-","-"] )
//                                            } else {
//                                                console.log( "onConnectionStatusChanged: BOUSE " + status )

//                                            }
//                                        }
//                                    }
//                                }

//                                //Label { text: "MC Configuration" }

//                                Text {
//                                    id: config1
//                                    text: qsTr(config1.buildText( ["-","-","-","-","-"] ))

//                                    function buildText(values) {
//                                        config1.text = "Version: " + (values[3]) + "." + (values[2]) + "." + (values[1]) + "\n"
//                                                        +"Motors: " + (values[4])


//                                        console.log("values=" + values)
//                                    }

//                                    McRegProxy {
//                                        id: config1Reg
//                                        name: "GLOBAL_CONFIG"

//                                        onValuesChanged: {
//                                            config1.buildText(values)
//                                            console.log( "UPDATED CONFIG: " + values )
//                                        }
//                                    }
//                                }

//                                Text {
//                                    id: config2
//                                    text: qsTr(config2.buildText( ["-","-","-","-","-"] ))

//                                    function buildText(values) {
//                                        config2.text = "Max App Speed:   " + (values[0]?values[0]:"?") + " RPM\n"
//                                                    + "Max App Current: " + (values[1]?values[1]:"?") + " A\n"
//                                                    + "Max App Voltage: " + (values[2]?values[2]:"?") + " V\n"
//                                                    + "Min App Voltage: " + (values[3]?values[3]:"?") + " V\n"
//                                                    + "Drive Type:      " + values[4]
//                                    }

//                                    McRegProxy {
//                                        id: config2Reg
//                                        name: "MOTOR_CONFIG"

//                                        onValuesChanged: {
//                                            config2.buildText( values )

//                                            //if this register is not correctly filled in FW value can be too high and qml take too much time (or crash)  to adapt gauge

//                                            console.log( "UPDATED MOTOR CONFIG: " + values )
//                                            if ( config2Reg.reg.values[0] < 200000 &&  config2Reg.reg.values[0] >0) {
//                                                surface.minimumSpeed = - config2Reg.reg.values[0];
//                                                surface.maximumSpeed = config2Reg.reg.values[0];
//                                            }


//                                            if ( config2Reg.reg.values[2] < 1000 &&  config2Reg.reg.values[2] >0) {
//                                                console.log( "new max Vbus =  " +  config2Reg.reg.values[2])
//                                                //calculate the closest ten of Vbus
//                                                var maxVbus =config2Reg.reg.values[2];
//                                                var num_digit = Math.round(maxVbus).toString().length;
//                                                var unit = Math.round(maxVbus).toString()[0]
//                                                var vbus_10 = (parseInt(unit,16) + 1) * Math.pow(10,num_digit-1)
//                                                surface.maximumVBus = vbus_10;
//                                                gaugeVerticalVbus.tickmarkStepSize = vbus_10 / 5

//                                            }
//                                        }
//                                    }
//                                }


//                                Text {
//                                    id: config3
//                                    text: qsTr(config3.buildText( ["-","-","-","-","-","-"] ))

//                                    function buildText(values) {
//                                        var t = "Primary speed sensor:   "

//                                        switch ( values[0] ) {
//                                        case 0: t += "No sensor"; break
//                                        case 1: t += "STO+PLL"; break;
//                                        case 2: t += "STO+Cordic"; break;
//                                        case 3: t += "Encoder"; break;
//                                        case 4: t += "Hall sensor"; break;
//                                        case 5: t += "HSO"; break;
//                                        case 6: t += "HSO+ZEST"; break;
//                                        default: t+= "-";break;
//                                        }

//                                        t+= "\nAux speed sensor:          "

//                                        switch ( values[1] ) {
//                                        case 0: t += "No sensor"; break
//                                        case 1: t += "STO+PLL"; break;
//                                        case 2: t += "STO+Cordic"; break;
//                                        case 3: t += "Encoder"; break;
//                                        case 4: t += "Hall sensor"; break;
//                                        case 5: t += "HSO"; break;
//                                        case 6: t += "HSO+ZEST"; break;
//                                        }

//                                        t += "\nCurrent sensing topology: "

//                                    switch( values[2] ) {
//                                    case 0: t += "3 Shunt"; break;
//                                    case 1: t += "1 Shunt"; break;
//                                    case 2: t += "1 Shunt, phase shift"; break;
//                                    case 3: t += "ICS"; break;
//                                    }

//                                    t += "\nFOC rate: " + values[3]
//                                    t += "\nPWM frequency: " + values[4]
//                                    t += "\nMedium frequency: " + values[5]
//                                    if ( values[6] & 1 ) t+= "\nFlux Weakening"
//                                    if ( values[6] & 2 ) t+= "\nFeed Forward"
//                                    if ( values[6] & 4 ) t+= "\nMTPA"
//                                    if ( values[6] & 8 ) t+= "\nPFC"
//                                    if ( values[6] & 16 ) t+= "\nICL"
//                                    if ( values[6] & 32 ) t+= "\nResistive Brake"
//                                    if ( values[6] & 64 ) t+= "\nOCP Disable"
//                                    if ( values[6] & 128 ) t+= "\nSTGAP"
//                                    if ( values[6] & 256 ) t+= "\nPosition Control"
//                                    if ( values[6] & 512 ) t+= "\nVBus Sensing"
//                                    if ( values[6] & 1024 ) t+= "\nTemperature Sensing"
//                                    if ( values[6] & 2048 ) t+= "\nVoltage Sensing"
//                                    if ( values[6] & 4096 ) t+= "\nFlash Config"
//                                    if ( values[6] & 8192 ) {
//                                        dac1RowLayout.enabled = true
//                                        t+= "\nDAC Channel 1"
//                                    }
//                                    else{
//                                        dac1RowLayout.enabled = false
//                                    }

//                                    if ( values[6] & 16384 ) {
//                                        dac2RowLayout.enabled = true
//                                        t+= "\nDAC Channel 2"
//                                    }
//                                    else{
//                                        dac2RowLayout.enabled = false
//                                    }

//                                    if ( values[7] & 1 ) t+= "\nMCP over STLink"
//                                    if ( values[7] & 2 ) t+= "\nMCPA over STLink"
//                                    if ( values[7] & 4 ) t+= "\nMCP over UARTA"
//                                    if ( values[7] & 8 ) t+= "\nMCPA over UARTA"
//                                    if ( values[7] & 16 ) t+= "\nMCP over UARTB"
//                                    if ( values[7] & 32 ) t+= "\nMCPA over UARTB"

//                                    mcRegBank.setFWConfig(t);
//                                    console.log( "UPDATED FOC FW CONFIG text= " + t )



//                                        config3.text = t

//                                    }

//                                    McRegProxy {
//                                        id: config3Reg
//                                        name: "FOC_FW_CONFIG"

//                                        onValuesChanged: {
//                                            config3.buildText( values )

//                                            console.log( "UPDATED FOC FW CONFIG: " + values )
//                                        }
//                                    }
//                                }
//                            }

//                        }
                    } // RowLayout
                    RevUpComponent {
                        id: revupComponent
                    }
                } // StackLayout
            } // ColumnLayout
            }

            GroupBox {
                title: "Advanced Configuration"
                id : advancedModeGroupBox

                visible: advancedModeButton.checked

                Layout.alignment: Qt.AlignTop

                ColumnLayout {

                    TabBar {
                        id: advancedTabBar

                        TabButton { text: "Currents & Speed"}
                        TabButton { text: "Observers" }

                        Layout.fillWidth: true
                    }

                    StackLayout {
                        currentIndex: advancedTabBar.currentIndex

                        Layout.fillWidth: true

                        ColumnLayout {
                            GroupBox {
                                id: piGroupBox

                                title: "Speed PI regulator"

                                ColumnLayout {

                                    McRegGroupSpinBox {
                                        RegDesc { regName: "SPEED_KP"; label: "Speed Kp" }
                                        RegDesc { regName: "SPEED_KI"; label: "Speed Ki" }
                                        //RegDesc { regName: "SPEED_KD" }
                                    }

                                    McRegComboBox {
                                        Layout.fillWidth: true

                                        regName: "SPEED_KP_DIV"
                                        label: "Kp divisor"
                                    }

                                    McRegComboBox {
                                        Layout.fillWidth: true

                                        regName: "SPEED_KI_DIV"
                                        label: "Ki divisor"
                                    }
                               }
                            } /* Speed PID */

                            GroupBox {
                                title: "Torque (Iq) PI regulator"

                                McRegGroupSpinBox {
                                    RegDesc { regName: "I_Q_KP"; label: "Torque Kp" }
                                    RegDesc { regName: "I_Q_KI"; label: "Torque Ki" }
                                    //RegDesc { regName: "TORQUE_KD" }
                                }
                            } /* Torque PID */

                            ToolSeparator { orientation: Qt.Horizontal }

                            McRegSpinBox { regName: "I_Q_REF"; label: "Torque Reference"; from: -32768; to: 32767 }

                            GroupBox {
                                title: "Flux (Id) PI"

                                McRegGroupSpinBox {
                                    RegDesc { regName: "I_D_KP"; label: "Flux Kp" }
                                    RegDesc { regName: "I_D_KI"; label: "Flux Ki" }
                                    //RegDesc { regName: "FLUX_KD" }
                                }
                            } /* Flux PID */

                            ToolSeparator { orientation: Qt.Horizontal }

                            McRegSpinBox { regName: "I_D_REF"; label: "Flux Reference"; from: -32768; to: 32767 }
                        } /* Speed & Current PID Column */


                        ColumnLayout {
                            GroupBox {
                                title: "State Observer with PLL"

                                Layout.fillWidth: true

                                McRegGroupSpinBox {
                                    RegDesc { regName: "STOPLL_C1"; label: "C1";  }
                                    RegDesc { regName: "STOPLL_C2"; label: "C2";  }
                                    RegDesc { regName: "STOPLL_KP"; label: "PLL Kp" }
                                    RegDesc { regName: "STOPLL_KI"; label: "PLL Ki" }
                                }
                            } /* Observer + PLL */

                            GroupBox {
                                title: "State Observer with CORDIC"

                                Layout.fillWidth: true

                                McRegGroupSpinBox {
                                    RegDesc { regName: "STOCORDIC_C1"; label: "C1"; }
                                    RegDesc { regName: "STOCORDIC_C2"; label: "C2"; }
                                }
                            } /* Observer + PLL */
                        } /* Observers Column */
                    } // StackLayout
                } // ColumnLayout
            } // Advanced Configuration
        } // RowLayout


       } // Frame

       Component.onCompleted: {
        mcRegBank.getRegisterByName( "FAULTS_FLAGS" ).setPolling( true )
        mcRegBank.getRegisterByName( "STATUS" ).setPolling( true )
        mcRegBank.getRegisterByName("REVUP_DATA").setPolling(true)
        mcRegBank.enablePolling( true )
            //console.log( "rampDuration font: " + rampDuration.font )


       }


} // ScrollView

